/* CG/solution/parts/cgp03.c */

/****************************************************************
 *                                                              *
 * This file has been written as a sample solution to an        *
 * exercise in a course given at the High Performance           *
 * Computing Centre Stuttgart (HLRS).                           *
 * It is made freely available with the understanding that      *
 * every copy of this file must include this header and that    *
 * HLRS takes no responsibility for the use of the enclosed     *
 * teaching material.                                           *
 *                                                              *
 * This software is partitioned into 16 tasks.                  *
 * Each task can be implemented by one group of the course      *
 * Files: CG/solution/cg_all.c - Solution                       *
 *        CG/skel/cg_01.c      - Skeletons that should be used  *
 *             .. cg_15.c        to implement each task         *
 *        CG/solution/cgp00.c  - Source part of each task       *
 *             ..     cgp16.c                                   *
 *                                                              *
 * Authors: Dr. Rolf Rabenseifner                               *
 *          Felix Triebel                                       *
 *          Michael Speck                                       *
 *          Felix Triebel                                       *
 *          Traugott Streicher                                  *
 *                                                              *
 * Contact: rabenseifner@hlrs.de                                *
 *                                                              *
 ****************************************************************/
                                                           /*EOH*/
/*>>>>>>>>>>>>>>>>>>>>>> begin of task 03 <<<<<<<<<<<<<<<<<<<<<<*/

/*********************** matrix routines ************************/

/*------------------------------
 * void alloc_A
 *
 * TASK: This function allocates the local part of the matrix A.
 *
 * REFERENCE: Slides 15, 22
 *
 * USED GLOBAL DATA:
 *  IN  int start             : begin of the local chunk
 *  IN  int end1              : end+1 of the local chunk
 *  OUT struct matrix_chunk A : the matrix A
 * USED FUNCTIONS: APPL: save_malloc
 */
void alloc_A(void)
{
        int I_loc; // local row index in matrix (slide 25)

        /* allocate memory for the rows of the matrix */
        /* A.row is an array of size (end1-start) */
        A.row = (struct matrix_row*)
                save_malloc(sizeof(struct matrix_row) * (end1-start));

        /* for each row set the number of elements to zero */
        for(I_loc=0; I_loc < (end1-start); I_loc++) A.row[I_loc].size = 0;
}

/*------------------------------
 * void Alloc_Matrix_Row
 *  IN/OUT struct matrix_chunk* A : heat matrix
 *  IN     int global_row         : global index of the row in the heat matrix
 *  IN     int Number_Of_Elements : number of elements in the row
 *
 * TASK: This function allocates a row for the matrix A.
 *       The number of elements in the row is given.
 *
 * REFERENCE: Slides 15, 22
 *
 * USED GLOBAL DATA:
 *  IN int my_rank : rank of the local process (only for error handling)
 *  IN int start   : start of the local chunk
 * USED FUNCTIONS: APPL: Row2Rank, save_malloc
 */
void Alloc_Matrix_Row(struct matrix_chunk* A, int global_row, int Number_Of_Elements)
{
        struct matrix_row *Row; // local

        /* check if the row is located on this process */
        if (Row2Rank(global_row) != my_rank) { printf("ERROR: Matrix row not local!\n"); abrt(); }

        /* allocate */ 
        Row = &(A->row[global_row-start]);
        Row->value = (double*)save_malloc(sizeof(double)*Number_Of_Elements);
        Row->J = (int*)save_malloc(sizeof(int)*Number_Of_Elements);
        Row->J_loc = (int*)save_malloc(sizeof(int)*Number_Of_Elements);
}

/*------------------------------
 * void Mat_Set_Value
 *  IN/OUT struct matrix_chunk* A : matrix
 *  IN     int I                  : index I
 *  IN     int J                  : index J
 *  IN     double v               : value
 *
 * TASK: Sets the matrix element located at I,J to the given value.
 *
 * REFERENCE: Slides 15, 22
 *
 * USED GLOBAL DATA:
 *  IN int start : start of the local chunk
 * USED FUNCTIONS: none
 */
void Mat_Set_Value(struct matrix_chunk* A, int I, int J, double v)
{
        struct matrix_row *Row;

        Row = &(A->row[I - start]);
        Row->value[Row->size] = v;
        Row->J[Row->size] = J;
        if ( (J>=start) && (J<end1) ) Row->J_loc[Row->size] = J - start;
        else                          Row->J_loc[Row->size] = -1; /*undefined*/
        (Row->size)++;
}

/*>>>>>>>>>>>>>>>>>>>>>> -end- of task 03 <<<<<<<<<<<<<<<<<<<<<<*/
