/* CG/solution/parts/cgp04.c */

/****************************************************************
 *                                                              *
 * This file has been written as a sample solution to an        *
 * exercise in a course given at the High Performance           *
 * Computing Centre Stuttgart (HLRS).                           *
 * It is made freely available with the understanding that      *
 * every copy of this file must include this header and that    *
 * HLRS takes no responsibility for the use of the enclosed     *
 * teaching material.                                           *
 *                                                              *
 * This software is partitioned into 16 tasks.                  *
 * Each task can be implemented by one group of the course      *
 * Files: CG/solution/cg_all.c - Solution                       *
 *        CG/skel/cg_01.c      - Skeletons that should be used  *
 *             .. cg_15.c        to implement each task         *
 *        CG/solution/cgp00.c  - Source part of each task       *
 *             ..     cgp16.c                                   *
 *                                                              *
 * Authors: Dr. Rolf Rabenseifner                               *
 *          Felix Triebel                                       *
 *          Michael Speck                                       *
 *          Felix Triebel                                       *
 *          Traugott Streicher                                  *
 *                                                              *
 * Contact: rabenseifner@hlrs.de                                *
 *                                                              *
 ****************************************************************/
                                                           /*EOH*/
/*>>>>>>>>>>>>>>>>>>>>>> begin of task 04 <<<<<<<<<<<<<<<<<<<<<<*/

#ifndef serial
/*------------------------------
 * int Add_Element_To_Halo
 *   IN int I:    global index of the element
 *   RETURNS int: position of the element in halo
 *
 * TASK: Add an element to halo_info, if it is not yet existing.
 *       Helper for Init_Vec_Halo(), see below.
 *
 * REFERENCE: Slides 16, 17 
 *
 * USED GLOBAL DATA:
 *  IN OUT struct halo_info
 * USED FUNCTIONS: APPL: enlarge_int_array
 */
int Add_Element_To_Halo(int I)
{
        int halo_pos;
 
        /* We might use the element x for multiple matrix elements */
        /* => Is I already in the halo? If yes, return existing halo_pos */
        for (halo_pos=0; halo_pos<halo_info.size; halo_pos++)
        {
                if (halo_info.I[halo_pos] == I)
                   return halo_pos;
        }
 
        /* Element not found => */
        /*  a) If not enough space, i.e. halo_info.size==halo_info.size_allocated,
               then enlarge array halo_info.I with enlarge_int_array() (see above) */ 
        if (halo_info.size_allocated == halo_info.size)
                enlarge_int_array(&halo_info.I, &halo_info.size_allocated);
        /*  b) Add it and return new halo_pos */
        halo_info.I[halo_info.size] = I;
        halo_info.size++;
        return (halo_info.size - 1);
}

/*------------------------------
 * void Init_Vec_Halo(void)
 *
 * TASK: Go through A and enter position of needed
 *       non-local vector-elements to halo_info.
 *
 * REFERENCE: Slides 16, 17, 22
 *
 * USED GLOBAL DATA:
 *  IN  int start           : begin of the local chunk
 *  IN  int end1            : end+1 of the local chunk
 *  IN struct matrix_chunk A: Rows contain columns of elements we need.
 *  OUT struct halo_info    : Describes position of non-local elements in the halo
 * USED FUNCTIONS: APPL: Add_Element_To_Halo
 */
void Init_Vec_Halo(void)
{
        int I_loc, j, J;

        /* initialize empty halo_info (i.e., with {0,0,NULL}) */ 
        halo_info.size = 0;
        halo_info.size_allocated = 0;
        halo_info.I = NULL;
        /* compute A.row[I_loc].J_loc[j] by using Add_Element_To_Halo(J) */
        for (I_loc=0; I_loc<end1-start; I_loc++)
        {
                for (j=0; j<A.row[I_loc].size; j++)
                {
                        J = A.row[I_loc].J[j];
                        if ((J<start) || (J>=end1))
                                A.row[I_loc].J_loc[j] = Add_Element_To_Halo(J) + end1-start;
                }
        }
}
#endif

/*>>>>>>>>>>>>>>>>>>>>>> -end- of task 04 <<<<<<<<<<<<<<<<<<<<<<*/
