/* CG/solution/parts/cgp05.c */

/****************************************************************
 *                                                              *
 * This file has been written as a sample solution to an        *
 * exercise in a course given at the High Performance           *
 * Computing Centre Stuttgart (HLRS).                           *
 * It is made freely available with the understanding that      *
 * every copy of this file must include this header and that    *
 * HLRS takes no responsibility for the use of the enclosed     *
 * teaching material.                                           *
 *                                                              *
 * This software is partitioned into 16 tasks.                  *
 * Each task can be implemented by one group of the course      *
 * Files: CG/solution/cg_all.c - Solution                       *
 *        CG/skel/cg_01.c      - Skeletons that should be used  *
 *             .. cg_15.c        to implement each task         *
 *        CG/solution/cgp00.c  - Source part of each task       *
 *             ..     cgp16.c                                   *
 *                                                              *
 * Authors: Dr. Rolf Rabenseifner                               *
 *          Felix Triebel                                       *
 *          Michael Speck                                       *
 *          Felix Triebel                                       *
 *          Traugott Streicher                                  *
 *                                                              *
 * Contact: rabenseifner@hlrs.de                                *
 *                                                              *
 ****************************************************************/
                                                           /*EOH*/
/*>>>>>>>>>>>>>>>>>>>>>> begin of task 05 <<<<<<<<<<<<<<<<<<<<<<*/

#ifndef serial
/*------------------------------
 * void Find_Used_Ranks
 *   IN  struct neighbor_struct* sendrecv_halo : Points to send_halo or recv_halo
 *   OUT int* num                              : Number of communication partners
 *                                               for sendimg or receiving
 *   OUT int** ranks                           : Array of their ranks
 *
 * TASK: This function finds our communication partners' ranks.
 *       Helper for Init_Recv_Info() and Init_Send_Info1(). 
 *
 * REFERENCE: Slide 18
 *
 * USED GLOBAL DATA:
 *   In int numprocs:           Number of MPI processes
 *   IN struct sendrecv_halo:   Describes position of non-local elements in the halo
 * USED FUNCTIONS: APPL: save_malloc
 */
void Find_Used_Ranks(struct neighbor_struct* sendrecv_halo, int* num, int** ranks)
{
        int rank, i, num_neighbors;
        num_neighbors = 0;

        /* calculate the number of neighbors (recv_num, send_num) */
        for (rank=0; rank<numprocs; rank++)
          if( sendrecv_halo[rank].size > 0)  num_neighbors++;

        /* allocate memory for recv_ranks, send_ranks */
        *ranks = (int*)save_malloc(sizeof(int)*num_neighbors);

        /* initialize the array recv_ranks, send_ranks */
        i=0;
        for (rank=0; rank<numprocs; rank++)
          if( sendrecv_halo[rank].size > 0)
          {
            (*ranks)[i] = rank;
            i++;
          }

        *num = num_neighbors;
}
#endif

#ifndef serial
/*------------------------------
 * void Init_Recv_Info
 *
 * TASK: This function sets up the recv_halo data-structure.
 *
 * REFERENCE: Slides 18, 20, 22
 *
 * USED GLOBAL DATA:
 *   IN struct halo_info:     Describes position of non-local elements in the halo
 *   In int numprocs:                           Number of MPI processes
 *   IN int start:                              start of the local chunk
 *   IN int end1:                               end + 1 of the local chunk
 *   OUT struct neighbor_struct* recv_halo:     Information about data we want to reveive
 *   OUT int recv_num:                          number of processors we receive data from
 *   OUT int* recv_ranks:                       ranks of all real neighbors that are sending halo-data
 * USED FUNCTIONS: APPL: save_malloc, Row2Rank, Find_Used_Ranks
 */
void Init_Recv_Info(void)
{
        int halo_pos, j, rank, size, local_index;

        /* allocate memory for recv_halo */ 
        recv_halo = (struct neighbor_struct*)save_malloc(
                        sizeof(struct neighbor_struct)*numprocs);

        /* for all processes set recv_halo.size to 0 */
        for (rank=0; rank<numprocs; rank++) recv_halo[rank].size = 0;

        /* count the elements to communicate for each rank */
        for (halo_pos=0; halo_pos < halo_info.size; halo_pos++)
        {
                rank = Row2Rank(halo_info.I[halo_pos]);
                recv_halo[rank].size++;
        }

        /* allocate memory for the recv_halo.scratch and recv_halo.I_loc
         * reset recv_halo.size to 0 */
        for (rank=0; rank<numprocs; rank++)
        {
                size=recv_halo[rank].size;
                recv_halo[rank].scratch
                        = (double*)save_malloc(sizeof(double)*size);
                recv_halo[rank].I_loc
                        = (int*)save_malloc(sizeof(int)*size);
                recv_halo[rank].size = 0;
        }


        /* initialize recv_halo.I_loc and recv_halo.size */
        for (halo_pos=0; halo_pos < halo_info.size; halo_pos++)
        {
                local_index = end1-start + halo_pos;
                rank = Row2Rank(halo_info.I[halo_pos]);
                j = recv_halo[rank].size; /* next free position */
                recv_halo[rank].I_loc[j] = local_index;
                recv_halo[rank].size++;
        }
        /* Use Find_Used_Ranks() to find all neighbors (i.e., ranks) from which halo-data is received */ 
        Find_Used_Ranks(recv_halo, &recv_num, &recv_ranks);
}
#endif

/*>>>>>>>>>>>>>>>>>>>>>> -end- of task 05 <<<<<<<<<<<<<<<<<<<<<<*/
