/* CG/solution/parts/cgp06.c */

/****************************************************************
 *                                                              *
 * This file has been written as a sample solution to an        *
 * exercise in a course given at the High Performance           *
 * Computing Centre Stuttgart (HLRS).                           *
 * It is made freely available with the understanding that      *
 * every copy of this file must include this header and that    *
 * HLRS takes no responsibility for the use of the enclosed     *
 * teaching material.                                           *
 *                                                              *
 * This software is partitioned into 16 tasks.                  *
 * Each task can be implemented by one group of the course      *
 * Files: CG/solution/cg_all.c - Solution                       *
 *        CG/skel/cg_01.c      - Skeletons that should be used  *
 *             .. cg_15.c        to implement each task         *
 *        CG/solution/cgp00.c  - Source part of each task       *
 *             ..     cgp16.c                                   *
 *                                                              *
 * Authors: Dr. Rolf Rabenseifner                               *
 *          Felix Triebel                                       *
 *          Michael Speck                                       *
 *          Felix Triebel                                       *
 *          Traugott Streicher                                  *
 *                                                              *
 * Contact: rabenseifner@hlrs.de                                *
 *                                                              *
 ****************************************************************/
                                                           /*EOH*/
/*>>>>>>>>>>>>>>>>>>>>>> begin of task 06 <<<<<<<<<<<<<<<<<<<<<<*/

#ifndef serial
/*------------------------------
 * void Init_Send_Info1(void)
 *
 * TASK: Broadcast number of elements we need from other
 *       processors (and reveive other processors info)
 *
 * REFERENCE: Slides 19, 20, 23
 *
 * USED GLOBAL DATA:
 *  In int numprocs:                     Number of MPI processes
 *  IN neighbor_struct* recv_halo:       Information about data we want to reveive
 *  OUT neighbor_struct* send_halo:      Information about data we have to send
 *  OUT int send_num:                    number of processors we send data to
 *  OUT int* send_ranks:                 ranks of all real neighbors to which halo-data must be sent
 * USED FUNCTIONS: APPL: Find_Used_Ranks
 *                 MPI : MPI_Alltoall
 */
void Init_Send_Info1(void)
{
        int rank;
        int *sendbuf, *recvbuf;

        /* allocate memory for sendbuf, recvbuf, and send_halo */ 
        sendbuf = (int*)save_malloc(sizeof(int)*numprocs);
        recvbuf = (int*)save_malloc(sizeof(int)*numprocs);

        send_halo = (struct neighbor_struct*)save_malloc(
                        sizeof(struct neighbor_struct)*numprocs);

        /* send to all processes number of requested elements */
        for (rank=0; rank<numprocs; rank++)
                sendbuf[rank] = recv_halo[rank].size;
        MPI_Alltoall(sendbuf, 1, MPI_INT, recvbuf, 1, MPI_INT, MPI_COMM_WORLD);
        for (rank=0; rank<numprocs; rank++)
                send_halo[rank].size = recvbuf[rank];

        /* Use Find_Used_Ranks() to find all neighbors (i.e.,ranks) that have to send halo-data */ 
        Find_Used_Ranks(send_halo, &send_num, &send_ranks);

        /* free scratch memory */ 
        free(sendbuf); free(recvbuf);
}
#endif

/*>>>>>>>>>>>>>>>>>>>>>> -end- of task 06 <<<<<<<<<<<<<<<<<<<<<<*/
