/* CG/solution/parts/cgp09.c */

/****************************************************************
 *                                                              *
 * This file has been written as a sample solution to an        *
 * exercise in a course given at the High Performance           *
 * Computing Centre Stuttgart (HLRS).                           *
 * It is made freely available with the understanding that      *
 * every copy of this file must include this header and that    *
 * HLRS takes no responsibility for the use of the enclosed     *
 * teaching material.                                           *
 *                                                              *
 * This software is partitioned into 16 tasks.                  *
 * Each task can be implemented by one group of the course      *
 * Files: CG/solution/cg_all.c - Solution                       *
 *        CG/skel/cg_01.c      - Skeletons that should be used  *
 *             .. cg_15.c        to implement each task         *
 *        CG/solution/cgp00.c  - Source part of each task       *
 *             ..     cgp16.c                                   *
 *                                                              *
 * Authors: Dr. Rolf Rabenseifner                               *
 *          Felix Triebel                                       *
 *          Michael Speck                                       *
 *          Felix Triebel                                       *
 *          Traugott Streicher                                  *
 *                                                              *
 * Contact: rabenseifner@hlrs.de                                *
 *                                                              *
 ****************************************************************/
                                                           /*EOH*/
/*>>>>>>>>>>>>>>>>>>>>>> begin of task 09 <<<<<<<<<<<<<<<<<<<<<<*/

/*------------------------------
 * double dot_product_row_vec
 * IN struct matrix_row* r:    Row of a matrix
 * IN vector_chunk v1:         Vector with halo
 *
 * TASK: Calculates dot product {row of matrix} * {vector_with_halo}.
 *       Helper for mult_A_vec()
 *
 * REFERENCE: Slides 14, 15, 16, 23
 *
 * USED GLOBAL DATA:
 * USED FUNCTIONS: none
 */
double dot_product_row_vec(struct matrix_row* r, vector_chunk v1)
{
        int j, J_loc;
        double sum=0;

        /* dot product of local chunk */
        for(j=0; j < r->size; j++)
        {
                /* get position */
                J_loc=r->J_loc[j];
                sum += r->value[j] * v1[J_loc];
        }

        return sum;
}

/*------------------------------
 * mult_A_vec
 *      IN  v1: vector_chunk v1
 *      OUT v2: vector_chunk
 *
 * TASK: Calculates dot product {row of matrix} * {vector}
 *       for all rows of the local chunk.
 *
 * REFERENCE: Slides 14, 15, 16, 23
 *
 * USED GLOBAL DATA: 
 *  IN start, end1
 * USED FUNCTIONS: APPL: dot_product_row_vec
 */
void mult_A_vec(vector_chunk v1, vector_chunk /* out */ v2)
{
        int I_loc;

        for(I_loc=0; I_loc < (end1-start); I_loc++)
                v2[I_loc] = dot_product_row_vec(&(A.row[I_loc]), v1);
}

/*>>>>>>>>>>>>>>>>>>>>>> -end- of task 09 <<<<<<<<<<<<<<<<<<<<<<*/
